/**************************************************************************************************************************
// This script was authored by Lorne Brooks - Email: lbtools2k@gmail.com
//
// A tool for visualising the projected lower and upper masses (pelvis and base of the neck respectively) in relation to the feet of a
// standard Biped rig. Markers between the feet show where the masses intersect, which may help with determining where the static
// weight is distributed when posing. It does not take into account motion dynamics and so is mainly used for when standing still.
// Additionally, when a foot is raised above the ground it's line markers turn blue. Likewise if lowered below the ground they turn red.
// Otherwise they stay green when level with the ground (+ or - 0.1 unit tollerance).
//
// To add a Biped Mass Marker to a Biped, simply select any single Biped part and click the toolbar icon. After the markers are imported
// into the scene, a selection set is created containing the frozen markers. To delete the markers and selection set, hold SHIFT and click
// the toolbar icon.
//
//
// REVISION HISTORY:
// Version 1.0 01/04/18;
//						- First public release.
//
//
// This script is not copyrighted, please feel free to modify or distribute.
//
**************************************************************************************************************************/

global g_LBTools_BMM

struct s_LBTools_bipedMassMarkers
(

/*==================================================================================
// Structure variables
====================================================================================*/
	
BMMmaxfile = getDir #plugcfg + @"\LBTools_BipedMassMarkers.max",
	
/*==================================================================================
// Structure functions
/*==================================================================================

f_addConstraints
f_findHierarchyRoot
f_deleteBMMlayer
f_bipedMassMarkers
	
====================================================================================*/

fn f_addConstraints sourceObj targetObj relative:false =
(
	sourceObj.pos.controller = Position_List()
	sourceObj.pos.controller.available.controller = Position_Constraint()
	sourceObj.pos.controller[2].appendTarget targetObj 50.0
	sourceObj.pos.controller.active = 2
	
	sourceObj.rotation.controller = Rotation_List()
	sourceObj.rotation.controller.available.controller = Orientation_Constraint()
	sourceObj.rotation.controller[2].appendTarget targetObj 50.0
	sourceObj.rotation.controller.active = 2
	
	if relative do
	(
		sourceObj.pos.controller.Position_Constraint.controller.RELATIVE = on
		sourceObj.rotation.controller.Orientation_Constraint.controller.RELATIVE = on
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_findHierarchyRoot obj =
(
	if obj != undefined do 
	(
		if isValidnode obj.parent then
		(
			f_findHierarchyRoot obj.parent
		)
		else
		(
			obj
		)
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_deleteBMMlayer =
(
	local layerSnapshots = layerManager.getLayerFromName "BIPED MASS MARKERS"
	if layerSnapshots != undefined do
	(
		layerSnapshots.nodes &nodesInLayer
		delete nodesInLayer
		layerManager.deleteLayerByName "BIPED MASS MARKERS"
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

/************************************************************************************************
// Note:
// For multiple Biped rigs in the scene, each one must have a unique root name for multiple BMMs.
*************************************************************************************************/
fn f_bipedMassMarkers =
(
	if doesFileExist g_LBTools_BMM.BMMmaxfile then
	(
		if selection.count == 1 AND classOf selection[1] == Biped_Object then
		(
			local selectedBipedRoot = g_LBTools_BMM.f_findHierarchyRoot selection[1]
			mergeMAXFile g_LBTools_BMM.BMMmaxfile #useMergedMtlDups #neverReparent #select
			
			-- // Check if the BMM helper already exist in the scene for this Biped
			local moduleExists = false
			for h in helpers do
			(
				if (findString h.name ("BMM_Floor_Pt" + "_" + selectedBipedRoot.name)) != undefined do
				(
					moduleExists = true
				)
			)
			
			if moduleExists == false then
			(			
				-- // Rename the BMM module to avoid naming conflicts with multiple characters, for expose TM assignment - based on Biped root name as the suffix
				local a_BMMobjs = #()
				a_BMMobjs = for s in selection collect s
				for s in a_BMMobjs do
				(
					s.name = s.name + "_" + selectedBipedRoot.name
				)
			
				-- // Assign the renamed BMM objects to common variables
				local BMM_Floor_Pt = undefined
				local BMM_Floor2_Pt = undefined
				local BMM_LFoot_Pt = undefined
				local BMM_RFoot_Pt = undefined
				local BMM_Pelvis_Pt = undefined
				local BMM_Ribcage_Pt = undefined
				local BMM_LFoot_ExpTM = undefined
				local BMM_RFoot_ExpTM = undefined
				local BMM_Pelvis_ExpTM = undefined
				local BMM_Ribcage_ExpTM = undefined
				local BMM_LFoot_Line = undefined
				local BMM_RFoot_Line = undefined
				local BMM_LowerMass_Line = undefined
				local BMM_UpperMass_Line = undefined
				local BMM_UpperToLower_Line = undefined
				local BMM_LFoot_MassLine = undefined
				local BMM_RFoot_MassLine = undefined
				local BMM_FootToFoot_Line = undefined
				local BMM_MassCentre_Dot = undefined
				local BMM_FootCentre_Dot = undefined
				
				for s in a_BMMobjs do
				(
					if (findString s.name "BMM_Floor_Pt") != undefined do (BMM_Floor_Pt = s)
					if (findString s.name "BMM_Floor2_Pt") != undefined do (BMM_Floor2_Pt = s)
					if (findString s.name "BMM_LFoot_Pt") != undefined do (BMM_LFoot_Pt = s)
					if (findString s.name "BMM_RFoot_Pt") != undefined do (BMM_RFoot_Pt = s)
					if (findString s.name "BMM_Pelvis_Pt") != undefined do (BMM_Pelvis_Pt = s)
					if (findString s.name "BMM_Ribcage_Pt") != undefined do (BMM_Ribcage_Pt = s)
					if (findString s.name "BMM_LFoot_ExpTM") != undefined do (BMM_LFoot_ExpTM = s)
					if (findString s.name "BMM_RFoot_ExpTM") != undefined do (BMM_RFoot_ExpTM = s)
					if (findString s.name "BMM_Pelvis_ExpTM") != undefined do (BMM_Pelvis_ExpTM = s)
					if (findString s.name "BMM_Ribcage_ExpTM") != undefined do (BMM_Ribcage_ExpTM = s)
					if (findString s.name "BMM_LFoot_Line") != undefined do (BMM_LFoot_Line = s)
					if (findString s.name "BMM_RFoot_Line") != undefined do (BMM_RFoot_Line = s)
					if (findString s.name "BMM_LowerMass_Line") != undefined do (BMM_LowerMass_Line = s)
					if (findString s.name "BMM_UpperMass_Line") != undefined do (BMM_UpperMass_Line = s)
					if (findString s.name "BMM_UpperToLower_Line") != undefined do (BMM_UpperToLower_Line = s)
					if (findString s.name "BMM_LFoot_MassLine") != undefined do (BMM_LFoot_MassLine = s)
					if (findString s.name "BMM_RFoot_MassLine") != undefined do (BMM_RFoot_MassLine = s)
					if (findString s.name "BMM_FootToFoot_Line") != undefined do (BMM_FootToFoot_Line = s)
					if (findString s.name "BMM_MassCentre_Dot") != undefined do (BMM_MassCentre_Dot = s)
					if (findString s.name "BMM_FootCentre_Dot") != undefined do (BMM_FootCentre_Dot = s)
				)
				
				-- // Rename the material based on Biped root name
				BMM_UpperMass_Line.material.name = "BMM - " + selectedBipedRoot.name
				
				-- // Find the needed Biped parts
				bPelvis = biped.getNode selectedBipedRoot #Pelvis
				bNeck = biped.getNode selectedBipedRoot #Neck
				bLFoot = biped.getNode selectedBipedRoot #lLeg link:3
				bRFoot = biped.getNode selectedBipedRoot #rLeg link:3
				
				-- // Add constraints to BMM objects, binding them to the Biped parts
				g_LBTools_BMM.f_addConstraints BMM_LFoot_Pt bLFoot
				g_LBTools_BMM.f_addConstraints BMM_RFoot_Pt bRFoot
				g_LBTools_BMM.f_addConstraints BMM_Pelvis_Pt bPelvis
				g_LBTools_BMM.f_addConstraints BMM_Ribcage_Pt bNeck
				
				-- // Zero-off the feet helper positions to the floor
				BMM_LFoot_Pt.pos.controller.Available.controller = Position_XYZ()
				BMM_LFoot_Pt.pos.controller.active = 3
				BMM_LFoot_Pt.pos.z = 0.0
				BMM_RFoot_Pt.pos.controller.Available.controller = Position_XYZ()
				BMM_RFoot_Pt.pos.controller.active = 3
				BMM_RFoot_Pt.pos.z = 0.0
					
				-- // Assign Biped objects to Expose Node
				BMM_LFoot_ExpTM.exposeNode = BMM_LFoot_Pt
				BMM_RFoot_ExpTM.exposeNode = BMM_RFoot_Pt
				BMM_Pelvis_ExpTM.exposeNode = selectedBipedRoot
				BMM_Ribcage_ExpTM.exposeNode = bNeck

				-- // Hide and freeze helpers, and unhide the visualizer markers
				hide a_BMMobjs
				freeze a_BMMobjs
				unhide BMM_LFoot_Line; unhide BMM_RFoot_Line
				unhide BMM_LowerMass_Line; unhide BMM_UpperMass_Line
				unhide BMM_LFoot_MassLine; unhide BMM_RFoot_MassLine
				unhide BMM_FootCentre_Dot; unhide BMM_MassCentre_Dot
				unhide BMM_UpperToLower_Line; unhide BMM_FootToFoot_Line
			)
			else
			(
				delete selection
				messageBox "Error: A Biped Mass Marker already exists for this Biped." title:""
			)
			
			select selectedBipedRoot
			
			-- // Update the floor point
			sliderTime = sliderTime + 1
			sliderTime = sliderTime - 1
		)
		else
		(
			messageBox "Error: Select a single Biped part first." title:""
		)
	)
	else
	(
		messageBox "Error: LBTools_BipedMassMarkers maxfile does not exist. Merge failed." title:""
	)
)

) --// end struct

g_LBTools_BMM = s_LBTools_bipedMassMarkers()


/*********************************************************************************************************************************************************************/
--// MACRO CALLS FOR UI
/*********************************************************************************************************************************************************************/

macroscript bipedMassMarkers
	category:"LB Tools"
	buttontext:"{ BMM }"
 	icon:#("LBTools_BipedMassMarkers",1)
	tooltip:"Biped Mass Markers 1.0"
(
	if keyboard.shiftPressed then
	(
		g_LBTools_BMM.f_deleteBMMlayer()
	)
	else
	(
		g_LBTools_BMM.f_bipedMassMarkers()
	)
)

/*********************************************************************************************************************************************************************/